/*
     this is the config.h file
     main idea is that you will find here all
     macro-definitions for configuring the spin-program
     it is not necessary to edit other files any more (Feb2000)
     - for auto tests an empty config.h for default macros is enough
 */
#ifndef _CFG_H		/* if defined by make -D_CFG_H, CM etc. set outside */
#define _CFG_H
#ifndef NN
#define NN       40	 /* maximum nn = number of sites/spins */
#define Nw	 (4*NN)	 /* maximum nw = number of bounds (square-J1-J2) */
#endif
#define CM(JJ,tJ,tU) JJ	  /* Heisenberg-,  t-J- or  Hubbard-model */
#define NEW  4		/* maximum number of eigenvalues to calculate */

/* now some tuning options (memory versus speed, see speed_mpi.gpl): */
/* tuning macros could also be set via Makefile DFLAGS */
/* bigger buffers for more speed, but never eat more than 10% of main memory
 *  so 90% will left for storing vector and matrix (check: size ./exe/spin)
 * - Warning: NZXMAX * (NAHX=NUM_AHEAD_LINES) must fit into integer (32bit)
 *   minimize max_num_NonZero_elements_per_line_X (p.e. 2*min(nu,nd)*bonds/site+1)
 *   usually NZXMAX can be smaller for bigger NAHX (because averaging nzx)
 *   samples: J1-LC40 NZXMAX=81 J1J2sq40=161 J1J2sq64=257 */
/* #define NZXMAX ((Nw+1)/2+1) */ 
/* - maximize NUM_AHEAD_LINES for better MPI-scaling, but size a.out < 10%mem
 *   NAHX must be 1..maxint/thxy_size, mpi_pckts=AH*38/n p.e. 1K..16M
 *   kautz: 500MB/core NAHX*NZX <= 2M p.e. NAHX=4K exe= (ToDo depends on NZX) */
/* #define NUM_AHEAD_LINES (1024) */ 
/* - SiCortex09-.5GHz-.4GB*6c:   AH=1K HB=1M  43MBexe test950*6 2017-01-14
 * - lrzGarching2012-1.6GB*16c: AH=16K HB=2M 400MBexe test32*16 2017-01-12
 * good_value: size_spin= 2*100kB*mpi_n, but limit to max 10%freeMem/task 
 *             assuming 90%++ max mpi_speed QDR-IB reached for 100kB # y2016  
 *  how to choose good Mem/Node-ratio? assuming QDR-IB, max speed at 100kB++ 
 *    Hardware.MinMem/Node=(Tasks/Node)^2*MaxNodes*2*100kB*1/10%
 *        p.e. 16^2*1000*2MB=512GB(pureMPI)  or 1*1000*2MB=2GB(MPI+PT)   
 *  for network = bottleneck and MPI is used asynchron (compute during wait)
 */
/* CFG_CPUSET should set for best speed since 2017-01 on SMP nodes,
 *            but not on MPI pinning conflicts compile errors (please report),
 *            bitpattern is not needed and ignored since 2017-01 */
/* #define CFG_CPUSET 2  */ /* 1=show 2=set pinning threads/tasks to cores */

#define STOREH 2
/* store matrix to disk 2=rw, 3=gz 4=buffered 5=bz2 0=memory(obsolete) */
/* v2.60 fills all available memory before using disk
 *   ToDo: file/node (now: file per CPU-core which is bad for most systems)
 *   set max available mem/disk in daten.i (maxmem, maxfile)
 */

/* #define CFG_FTLM 1 */ /* 2016-12 finite temperature lanczos method */

/* if matrix does not fit to memory, all memory will be eaten for best perf
 *   but some reserve is needed to let MPI or checkpointing work
 *   kautz=5800cores/18*64kB=21MB + 4MB for checkpointing (2015-04)
 *   comp2 22*16cores*4MB=1408MB ompi-1.8 (untested) */
/* #define ReserveMB 26 */ /* this memory will not be eaten by storeH */

#undef CONFIG_ANSATZ    /* define/undef CONFIG_ANSATZ for using this! */

#define VecType  5      /* undef or define to 0, 1, 2, 4, 5, 8 */
/* 2017-02-15 new dflt 5, old 0, warn wasting size, instead skip cplx-syms
 *     but high end big computations should made using VecType 0 or 4
 * 0=float(32bit),1=double(64bit),2=long dbl(80bit),8=flt16(16bit)   
 * 4=float-...   ,5=double-...    complex vectors (c++ or c99 needed)
 * check if float4 is enough accuracy, earn partly double speed by SIMD ...
 *     and double speed on slow networks (ToDo: dynamic adaption?)
 */

/* --- ABEL(NOABEL) use of Abel(non-Abel) symmetries --- */
#define CONFIG_NOABEL 1		/* CONFIG_ABEL or CONFIG_NOABEL */

/* --- S1SYM - special handling for composed spins (Lieb-Mattis-SL) --- */
/*             faster on 2s=2,3,4... systems but slower for 2s=1 systems */
#define CONFIG_S1SYM  1		/* CONFIG_S1SYM or CONFIG_NOS1SYM */
/*
 *  CONFIG_S1SYM useful for spin-1 systems with a lot of symmetries
 *             - does reduce the time needed for scanning for symmetries
 *             - this option should vanish in later versions (js Okt.98)
 */
/* sometimes symmetry search is so slow that we want to switch it off */
#define CONFIG_SymSearch 1   /* default: 1 = searching symmetries */   
/*
 * #define HFIELD if you want to have a magnetic field on each site
 * #undef  HFIELD if you do not need a magnetic field, its a bit faster
 */
#undef HFIELD
/*
 * #define SSANISO if you want to have a single-site anisotropy (by R. Schmidt)
 * #undef  SSANISO if you do not need single site anisotropy (runs faster)
 */
#undef SSANISO
/*
 * #define CONFIG_TBC if you want to have a TBC (twisted boundary conditions)
 * #undef  CONFIG_TBC if you do not need TBC, its faster
 */
#undef CONFIG_TBC

/* for large numbers you may have to enlarge stack size to avoid SEGFAULTs */
#define MaxSym   416      /* adjust it, takes S_int*(3*NN+5) memory, mod32! */

#endif
