#include <stdio.h>
#include <stdlib.h>
/*
   DSYEV - compute all eigenvalues and, optionally, eigenvectors of a real
           symmetric matrix A (LAPACK)
   ZHEEV - compute all eigenvalues and, optionally, eigenvectors of a complex
           Hermitian matrix A
   Tru64: cc -migrate lapack_test.c -ldxml # not_parallel
   IRIX:  cc lapack_test.c -lscs[_i8][_mp] # [integer8][multi-processor]
   linux: cc -DF2C lapack_test.c -llapack -lf2c  # lapack-3.0 + blas-3.0 (SuSE-8.0)
          cc -DF2C lapack_test.c -llapack -lf2c -lblas -lm -static # nonshared
   SuSE-9.1:   http://rpmseek.com/; rpm -i lapack-3.0-916.src.rpm
          cd /usr/src/packages/SOURCES;tar -xjf lapack-3.0.tar.bz2
          tar -xjf manpages.tar.bz2;cd lapack-3.0;patch -p0 <../lapack-3.0.dif
          make; cp liblapack.{a,so.3.0.0} /usr/lib/
          ln -s /usr/lib/liblapack.so.3.0.0 /usr/lib/liblapack.so.3
   SuSE-10.0-X86-64: lapack-3.0-926=/usr/lib/liblapack.so.3.0.0 32bit
   SuSE-10.3-X86-64:
   SL-6.5-2013-x86_64: cc -DDBL=0 -g -llapack /tmp/lapack_test.c # quanta2

     cc -DF2C -DDBL=0 -g -llapack -lf2c -Wall ../src/lapack_test.c
     cc       -DDBL=0 -g -llapack       -Wall ../src/lapack_test.c
     cc       -DDBL=0 -g -lopenblas -lpthread -Wall ../src/lapack_test.c
     OMP_NUM_THREADS=1 ./a.out 1000
     + valgrind test
 */
#ifndef DBL
#define DBL 1   /* 1=DBL*8 C, 0=CPLX*16 C++ */
#endif
#ifdef __cplusplus
extern "C" /* avoid special (C++)-names in object files, ex: main__Fv */
#endif
#if DBL
void dsyev_(
        const char *jobz,       /* (in) "N"=NoVectors, "V"=Vectors */
        const char *uplo,       /* (in) "U"=UpperTriangleA, "L"=LowerTA */
        const int *n,           /* (in) order of matrix */
        double *a,              /* a[n][lda] (in/out) */
        const int *lda,         /* (inp) leading dimension >=max(1,n) */
        double *w,              /* w[n] (out) eigenvalues */
        double *work,           /* work[lwork] (workspace/out) */
        const int *lwork,       /* (in)   >= max(1,3*n-1) better (nb+2)*n */
        int *info               /* (out) ==0 on success */
        );
#else
void zheev_(
        const char *jobz,       /* (in) "N"=NoVectors, "V"=Vectors */
        const char *uplo,       /* (in) "U"=UpperTriangleA, "L"=LowerTA */
        const int *n,           /* (in) order of matrix */
        double *a,              /* a[n][lda][2] (in/out), diagonal part I=0 */
        const int *lda,         /* (inp) leading dimension >=max(1,n) */
        double *w,              /* w[n] (out) eigenvalues */
        double *work,           /* work[lwork][2] (workspace/out) */
        const int *lwork,       /* (in)   >= max(1,2*n-1) better (nb+2) */
        double *rwork,          /* rwork[max(1,3N-2)] workspace 0..? 1..? */
        int *info               /* (out) ==0 on success */
        /* nb = blocksize for ZHETRD returned by ILAENV = 32 */
        );
#endif

/* system   n1   dsyev zheev result 
   wf       1000   24s   81s -4658.990435..125203.120225, dsyev im cache?
   wf       1500  133s  262s -5656.412444..187595.155427 
   wf       2000  283s  702s -6577.761139..250259.319939 
   wf       2500  627s 1285s
   wf       3000 1038s 2375s
   wf       3500 1764s 3466s
  => time[s] = 4e-8 * n1^3 (wf: GS160-alpha-731MHz dsyev) 
  => time[s] = 8e-8 * n1^3 (wf: GS160-alpha-731MHz zheev)
*/

int main(int argn, char *argv[]){
  int n1=2, lda, lwork, info=-1, i, j, i3, /* n1 = matrix size per option */
      cf=2-DBL,   /* cf: dbl=1, cplx=2 */
      nev=1,      /* flag eigenvectors */
      nb=32;      /* optimum block size, have seen 32 or 60 but dt=2% */
  double *a, *w, *work, *rwork;
  setvbuf(stdout, (char *)NULL, _IONBF, 0); /* no buffer */
  if (argn>1) {n1=atoi(argv[1]);printf(" set n1=%d\n",n1);}
  lda=n1;          /* matrix dimension */
  if (nb<3) { printf(" nb must be 3 or greater\n");exit(1);}
  lwork=(nb+1)*n1;     /* (empirical) factor (32+1) for working space */
  /* int-overflow check added 2014-03 */
  if (  ((nb+1)   *n1*cf)*sizeof(double)
      !=((nb+1)*1.*n1*cf)*sizeof(double)){printf("lwork overflow");exit(1);}
  if (  (cf   *n1*n1)
      !=(cf*1.*n1*n1)){printf("n1*n1 int overflow");exit(1);}
  if (  (cf   *n1*n1*sizeof(double))
      !=(cf*1.*n1*n1*sizeof(double))){printf("n1*n1*cplx overflow");exit(1);}
  /* allocating necessary memory malloc(size_t) */
  a=    (double *)malloc(cf*n1*n1*sizeof(double));
  w=    (double *)malloc(      n1*sizeof(double));
  work= (double *)malloc(cf*lwork*sizeof(double));
  if ((!work)||(!a)||(!w)) { printf(" malloc *work failed\n");exit(1); }
  printf(" n1=%d  matrix= %8.6e MiB\n", n1,
                             n1*1.*n1*cf*sizeof(double)/1024/1024);
  printf(" lwork= %d = %d * n1 + %d\n", lwork, lwork/n1, lwork%n1);
  /* now generating a pseudo-random matrix for test and diagonalize it */
#if DBL
  /* rand() is slow !*/
  for(i=0;i<n1;i++)for(j=0;j<=i;j++) a[i*n1+j]=a[j*n1+i]=0.0;
  for(i=0;i<n1;i++)for(j=0;j<64;j++) { /* 16 NZ per line */
    i3=(rand()%n1);
    a[i*n1+i3]=a[i3*n1+i]=(i3&255)-2.5;
  }
  for(i=0;i<lwork;i++)work[i]=170.6640625; /* special pattern 0xAA.AA */
  dsyev_("V","U",&n1,a,&lda,w,work,&lwork,&info);
#else
  rwork=(double *)malloc(  lwork*sizeof(double));  /* ??? 33*n1 > 3*n1-2 */
  if (!rwork) { printf(" malloc *rwork failed\n");exit(1); }
  /* ToDo: fill rwork spec. pattern and check return != pattern (debug) */
  for(i=0;i<  lwork;i++)rwork[i]=170.6640625; /* special pattern 0xAA.AA */
  for(i=0;i<2*lwork;i++) work[i]=170.625;    /* special pattern 0xAA.A0 */
  for(i=0;i<n1;i++)for(j=0;j<2*n1;j++) a[i*n1+j]= 0.0;
  for(i=0;i<n1;i++)for(j=0;j<64;j++){  /* 64 NZ per line */
    i3=(rand()%n1);
    a[(i*n1+i3)*2  ]= (a[(i3*n1+i)*2  ]=(i3&7)-4);
    a[(i*n1+i3)*2+1]=-(a[(i3*n1+i)*2+1]=(i3&2)-1);
  }
  {  
    int lwork=-1; /* output int via cplx array!? YES! result=(32+1)*n1 */
    zheev_(((nev)?"V":"N"),"U",&n1,a,&lda,w,work,&lwork,rwork,&info);
    printf(" zheev: optimal workspace size = %g = %d * n1 + %g\n",
           work[0], (int)(work[0]/n1), work[0]-n1*((int)(work[0]/n1)));
  }
  zheev_("V","U",&n1,a,&lda,w,work,&lwork,rwork,&info);
  for(i=0;i<lwork;i++) if(rwork[i]!=170.6640625)j=i; /* pattern 0xAA.AA */
  printf(" debug: last modified rwork index = %d\n", j);
#endif
  for(i=0;i<lwork;i++) if(work[i]!=170.625)j=i; /* pattern 0xAA.A0 */
  printf(" debug: last modified  work index = %d\n", j);
  printf(" info= %d\n w=",info);
  for(i=0;i<n1;i++) {
    if (i<16 || i>=n1-8) printf(" %f",w[i]); else
    if (i==16) printf(" ...");
  }
#ifdef EV    /* lot of output, only if you want it */
  for(i=0;i<n1 && i<4;i++){
    printf("\n v%d=",i);
    for(j=0;j<n1;j++){
#if DBL
      printf(" % f",a[i*n1+j);
#else
      printf(" (% f, % f)",a[(i*n1+j)*2],a[(i*n1+j)*2+1]);
#endif
    }
  }
#endif
  printf("\n");
  return 0;
}

#ifdef F2C
/* lapack needs f2c and f2c needs MAIN__() */
/* if you know a better solution, please tell me */
#ifdef __cplusplus
extern "C" /* avoid special (C++)-names in object files, ex: MAIN____Fv */
#endif
int MAIN__(int argn,char *argv[]){ return main(argn,argv); }
#endif
